#include "histogram.h"
#include <vector>
#include <numeric>
using namespace std;

void find_minmax(const vector<double>& numbers, double& min, double& max) {
    if (numbers.empty()) return;
    max = numbers[0];
    min = numbers[0];
    for (double x : numbers) {
        if (x < min) min = x;
        if (x > max) max = x;
    }
}

vector<size_t> make_histogram(vector<double> numbers, size_t bin_count) {
    double min, max;
    find_minmax(numbers, min, max);
    double bin_size = (max - min) / bin_count;
    vector<size_t> bins(bin_count);

    for (size_t i = 0; i < numbers.size(); i++) {
        bool found = false;
        for (size_t j = 0; (j < bin_count - 1) && !found; j++) {
            auto lo = min + j * bin_size;
            auto hi = min + (j + 1) * bin_size;
            if ((lo <= numbers[i]) && (numbers[i] < hi)) {
                bins[j]++;
                found = true;
            }
        }
        if (!found) {
            bins[bin_count - 1]++;
        }
    }
    return bins;
}

double calculate_average_height(const vector<size_t>& bins) {
    if (bins.empty()) return 0.0;
    double sum = accumulate(bins.begin(), bins.end(), 0.0);
    return sum / bins.size();
}
